﻿using log4net;
using Buddy.Common;
using System.ComponentModel;

namespace CombatBotEx {

    class CombatBotExConfig : JsonSettings {

        private ILog Log = LogManager.GetLogger("CombatBotExConfig");

        #region Singleton Implementation
        
        private static CombatBotExConfig _instance;

        public static CombatBotExConfig Instance {
            get { return _instance ?? (_instance = new CombatBotExConfig()); }
        } 

        #endregion

        #region Constructor
        
        public CombatBotExConfig()
            : base(GetSettingsFilePath(SettingsPath, "CombatBotEx", Buddy.Wildstar.Game.GameManager.LocalPlayer.Name, string.Format("{0}.json", "CombatBotExConfig"))) {
        } 
        
        #endregion

        #region SettingsVersion Binding

        private string _SettingsVersion;
        [DefaultValue("")]
        public string SettingsVersion {
            get { return _SettingsVersion; }
            set {
                if (value.Equals(_SettingsVersion)) {
                    return;
                }
                _SettingsVersion = value;
                NotifyPropertyChanged(() => SettingsVersion);
            }
        }

        #endregion

        #region SettingsAutoCollect Binding

        private bool _SettingsAutoCollect;
        [DefaultValue(true)]
        public bool SettingsAutoCollect {
            get { return _SettingsAutoCollect; }
            set {
                if (value.Equals(_SettingsAutoCollect)) {
                    return;
                }
                Log.Info("SettingsAutoCollect(" + value + ")");
                _SettingsAutoCollect = value;
                NotifyPropertyChanged(() => SettingsAutoCollect);
            }
        }

        #endregion

        #region SettingsAutoLoot Binding

        private bool _SettingsAutoLoot;
        [DefaultValue(true)]
        public bool SettingsAutoLoot {
            get { return _SettingsAutoLoot; }
            set {
                if (value.Equals(_SettingsAutoLoot)) {
                    return;
                }
                Log.Info("SettingsAutoLoot(" + value + ")");
                _SettingsAutoLoot = value;
                NotifyPropertyChanged(() => SettingsAutoLoot);
            }
        }

        #endregion

        #region SettingsInGameMessages Binding

        private bool _SettingsInGameMessages;
        [DefaultValue(true)]
        public bool SettingsInGameMessages {
            get { return _SettingsInGameMessages; }
            set {
                if (value.Equals(_SettingsInGameMessages)) {
                    return;
                }
                Log.Info("SettingsInGameMessages(" + value + ")");
                _SettingsInGameMessages = value;
                NotifyPropertyChanged(() => SettingsInGameMessages);
            }
        }

        #endregion

        #region SettingsHotkey Binding

        private bool _SettingsHotkey;
        [DefaultValue(true)]
        public bool SettingsHotkey
        {
            get { return _SettingsHotkey; }
            set
            {
                if (value.Equals(_SettingsHotkey))
                {
                    return;
                }
                Log.Info("SettingsHotkey(" + value + ")");
                _SettingsHotkey = value;
                NotifyPropertyChanged(() => SettingsHotkey);
            }
        }

        #endregion

        #region SettingsAutoAttack Binding

        private bool _SettingsAutoAttack;
        [DefaultValue(true)]
        public bool SettingsAutoAttack {
            get { return _SettingsAutoAttack; }
            set {
                if (value.Equals(_SettingsAutoAttack)) {
                    return;
                }
                Log.Info("SettingsAutoAttack(" + value + ")");
                _SettingsAutoAttack = value;
                NotifyPropertyChanged(() => SettingsAutoAttack);
            }
        }

        #endregion

        #region SettingsAutoFace Binding

        private bool _SettingsAutoFace;
        [DefaultValue(true)]
        public bool SettingsAutoFace {
            get { return _SettingsAutoFace; }
            set {
                if (value.Equals(_SettingsAutoFace)) {
                    return;
                }
                Log.Info("SettingsAutoFace(" + value + ")");
                _SettingsAutoFace = value;
                NotifyPropertyChanged(() => SettingsAutoFace);
            }
        }

        #endregion

        #region SettingsCheckHealth Binding

        private bool _SettingsCheckHealth;
        [DefaultValue(true)]
        public bool SettingsCheckHealth {
            get { return _SettingsCheckHealth; }
            set {
                if (value.Equals(_SettingsCheckHealth)) {
                    return;
                }
                Log.Info("SettingsCheckHealth(" + value + ")");
                _SettingsCheckHealth = value;
                NotifyPropertyChanged(() => SettingsCheckHealth);
            }
        }

        #endregion

        #region SettingsCheckLevel Binding

        private bool _SettingsCheckLevel;
        [DefaultValue(true)]
        public bool SettingsCheckLevel {
            get { return _SettingsCheckLevel; }
            set {
                if (value.Equals(_SettingsCheckLevel)) {
                    return;
                }
                Log.Info("SettingsCheckLevel(" + value + ")");
                _SettingsCheckLevel = value;
                NotifyPropertyChanged(() => SettingsCheckLevel);
            }
        }

        #endregion
    }
}

// WPF Data Binding: https://msdn.microsoft.com/en-us/library/ms752347(v=vs.100).aspx